/* gretl - The Gnu Regression, Econometrics and Time-series Library
 * Copyright (C) 1999-2000 Ramu Ramanathan and Allin Cottrell
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this software; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* utils.h for gretl */

#ifndef UTILS_H
#define UTILS_H

#include <stdio.h>

typedef struct {
    double *r;
    int nvar;
    int errcode;
} CORRMAT;

enum flagvals {
    OPT_O = 1,
    OPT_M,
    OPT_C,
    OPT_R,
    OPT_S,
    OPT_L,
    OPT_Z
};

/* functions follow */
 
double corr (const int n, const double *zx, const double *zy);

double covar (const int n, const double *zx, const double *zy);

double date (const int nt, const int pd, const double sd0);

int ftoi (const double f);

int ijton (const int i, const int j, const int lo);

int isdummy (const int varnum, const int t1, const int t2, 
	     const double *Z, const int n);

int iszero (const int t1, const int t2, const double *x);

void list_exclude (const int n, int *list);

int isconst (const int t1, const int t2, const double *x);

double esl_mean (const int t1, const int t2, const double *x);

void minmax (const int t1, const int t2, const double zx[], 
	     double *min, double *max);

int hasconst (const int *list);

int compare_doubles (const void *a, const void *b);

double esl_stddev (const int t1, const int t2, const double *x);

double esl_variance (const int t1, const int t2, const double *x);

void printlist (const int *list, const char *msg);

void aicetc (MODEL *pmod);

void criteria (const double ess, const int nobs, const int ncoeff, 
	       print_t *prn);

CORRMAT corrlist (int *list, const double *Z, const DATAINFO *pdinfo);

int adjust_t1t2 (MODEL *pmod, const int *list, int *t1, int *t2, 
		 const double *Z, const int n, int *misst);

int set_sample_dummy (const char *line, double **pZ, DATAINFO *pdinfo, 
		      char *msg);

int set_sample (const char *line, DATAINFO *pdinfo, char *msg);

int set_obs (char *line, DATAINFO *pdinfo, char *msg);

char *addpath (char *filename, PATHS *ppaths, int script);

int getopenfile (const char *line, char *fname, PATHS *ppaths,
		 int setpath, int script);

char getflag (int opt);

int catchflag (char *line, int *oflag);

int list_dups (const int *list, int ci);

void init_model (MODEL *pmod);

int free_model (void *ptr, SESSION *psession, session_t *rebuild);

void show_paths (PATHS *ppaths);

int set_paths (PATHS *ppaths, const int reset, const int gui);

int copylist (int **target, const int *src);

int grow_nobs (const int newobs, double **pZ, DATAINFO *pdinfo);

int grow_Z (const int newvars, double **pZ, DATAINFO *pdinfo);

int shrink_Z (const int delvars, double **pZ, DATAINFO *pdinfo);

int copyvec (double **targ, const double *src, const int n);

int swap_models (MODEL **targ, MODEL **src);

int forecast (int t1, const int t2, const int nv, 
	      const MODEL *pmod, DATAINFO *pdinfo, double **pZ);

int full_model_list (MODEL *pmod, int **plist);

double getvalue (const char *s, const double *Z, const DATAINFO *pdinfo);

int fcast_with_errs (const char *str, const MODEL *pmod, 
		     DATAINFO *pdinfo, double **pZ, print_t *prn,
		     const PATHS *ppaths, const int plot, char *msg);

int is_model_cmd (const char *line);

int save_model_spec (MODEL *pmod, char *buf);

int re_estimate (char *model_spec, MODEL *tmpmod, DATAINFO *pdinfo, 
		 double **pZ);

#endif /* UTILS_H */
